void readData()
{
  float startTime = millis();
  readSql();
  getDeweyDist();
  
  float endTime = millis();

  println("data loaded in " + (endTime - startTime)/1000 + " sec");
}

void readSql() {
  // connecting to the database
  msql = new MySQL( this, "tango.mat.ucsb.edu:3306", "spl1", "mat259", "V1sual1zat1on" ); //host, database, user, password

  while (msql.connect ()==false) {
    delay(1000);
  }

  if ( msql.connect() )
  {
    // for 3 keywords (2009-2012)
    for (int kw = 0; kw < 3; kw++) {

      // create data containers
      ArrayList tArray = new ArrayList(); // temporary ArrayList
      hmap = new HashMap();

      String query = "SELECT "+ 
        "SUM(CASE WHEN dewey.dewey BETWEEN 0 AND 99 THEN 1 ELSE 0 END) as d0," +
        "SUM(CASE WHEN dewey.dewey BETWEEN 100 AND 199 THEN 1 ELSE 0 END) as d1," +
        "SUM(CASE WHEN dewey.dewey BETWEEN 200 AND 299 THEN 1 ELSE 0 END) as d2," +
        "SUM(CASE WHEN dewey.dewey BETWEEN 300 AND 399 THEN 1 ELSE 0 END) as d3," +
        "SUM(CASE WHEN dewey.dewey BETWEEN 400 AND 499 THEN 1 ELSE 0 END) as d4," +
        "SUM(CASE WHEN dewey.dewey BETWEEN 500 AND 599 THEN 1 ELSE 0 END) as d5," +
        "SUM(CASE WHEN dewey.dewey BETWEEN 600 AND 699 THEN 1 ELSE 0 END) as d6," +
        "SUM(CASE WHEN dewey.dewey BETWEEN 700 AND 799 THEN 1 ELSE 0 END) as d7," +
        "SUM(CASE WHEN dewey.dewey BETWEEN 800 AND 899 THEN 1 ELSE 0 END) as d8," +
        "SUM(CASE WHEN dewey.dewey BETWEEN 900 AND 999 THEN 1 ELSE 0 END) as d9," +
        "count(*), DATE_FORMAT(o,'%Y-%m') as coutmonth " +
        "FROM title, activity, dewey " + 
        "WHERE title.bib = activity.bib AND activity.bib = dewey.bib AND LOWER(title) like '%" +
        keywords[kw] +
        "%' AND year(o) > 2008 AND year(o) < 2013 GROUP BY coutmonth;";

      // println(query);
      msql.query(query);
      //      msql.next();

      int count = 0;

      while ( msql.next () ) {

        // insert each result into a hashmap      
        String label = msql.getString(12);
        int total = msql.getInt(11);
        totalCount[kw] += total;

        int[] deweys= new int[10];
        for (int i = 0; i < 10; i++) {
          deweys[i] = msql.getInt(i+1);
          hmap.put(kw + "-" +label+"-"+i, new Integer(msql.getInt(i+1))); // (YYYY-MM-DW, ##) pair in hashmap
        }
        hmap.put(kw + "-" +label+"-t", new Integer(msql.getInt(11))); // (YYYY-MM, ##) pair in hashmap

        String[] temp = split(label, '-'); // Y-m

        // store into an array in Entity object
        tArray.add(new Entity(label, int(temp[0]), int(temp[1]), total, deweys));
        count++;
      }

      // fill out the missing items
      for (int i = startYear; i < (endYear+1); i++) { //years
        int annualSum = 0;

        for (int j = 1; j < 13; j++) { //months
          String label = kw + "-" + i + "-" + j + "-t";
          Integer n = (Integer)hmap.get(label);
          annualSum += (n!=null ? n : 0);

          // manual input for missing months
          if (kw==1 && i==startYear && j<3) {
            int[] temp = {
              0, 0, 0, 0, 0, 0, 0, 0, 0, 0
            };
            String templabel = (startYear)+"-0"+j;
            tArray.add(0, new Entity(templabel, endYear, j, 0, temp));
          }
        }
        annualCount[i-startYear] = annualSum;
      }
      mArray.add(new ArrayList(tArray));
      numMonth = tArray.size();
      tArray.clear();
    }
  }
}

// compute dewey distribution on each month across 3 topics
void getDeweyDist() {
  hmapDewey = new HashMap();
  hmapMonth = new HashMap();

  // temporary arrays
  ArrayList array1 = new ArrayList(mArray.get(0));
  ArrayList array2 = new ArrayList(mArray.get(1));
  ArrayList array3 = new ArrayList(mArray.get(2));

  for (int i = 0; i < numMonth; i++) {
    Entity e1 = (Entity) array1.get(i);
    Entity e2 = (Entity) array2.get(i);
    Entity e3 = (Entity) array3.get(i);
    float total = float(e1.total + e2.total + e3.total);
    hmapMonth.put(i, new Float(total));

    for (int j = 0; j < 10; j++) {
      String label = i + "-" + j;
      float val = float(e1.deweys[j] + e2.deweys[j] + e3.deweys[j]);
      //val = (val/float(total))*TWO_PI;
      hmapDewey.put(label, new Float(val));
    }
  }
}

